# -*- coding: utf-8 -*-
# Default libs

# Library modules
from pyunicode import safely_encode


class ExportFormat:
  ''' Supported formats '''
  PNG = 'png'
  PDF = 'pdf'


class ExportMimeType:
  ''' Supported mimetypes '''
  PNG = 'image/png'
  PDF = 'application/pdf'


class GenericObject(object):
  ''' Base class for any generic object '''
  def __unicode__(self):
    return u'<GenericObject>'

  def __str__(self):
    return safely_encode(unicode(self))

  def __repr__(self):
    try:
      u = str(self)
    except (UnicodeEncodeError, UnicodeDecodeError):
      u = '[Bad Unicode data]'
    return '<%s: %r>' % (self.__class__.__name__, u)


class Exportable(GenericObject):
  ''' Any exportable subcomponent of a file. e.g. layers, pages, artboards '''
  def export(self, export_format=ExportFormat.PNG):
    ''' Overwritten by subclasses to export themselves '''
    pass



# -*- coding: utf-8 -*-
# Default libs
import logging
import os

from glob import glob
from tempfile import mkdtemp

# Project modules
from ..settings import SKETCHTOOL
from ..utils import execute

logger = logging.getLogger(__name__)


############################################################
# EXPORT COMMANDS - PASSTHROUGH TO SKETCHTOOL EXPORT
############################################################
def export_cmd(cmd, src_path, dest_dir=None, item_id=None, export_format=None, scale=None):
  ''' Executes a `sketchtool export` command and returns formatted output

  :src_path: File to export. :type <str>
  :dest_dir: Items are exported at /dest_dir/name@scale.export_format e.g. `~/Desktop/Page 1@2x.png`
  :param export_format: 'png', 'pdf' etc. :type <ExportFormat>
  :param scale: Specify as 1.0, 2.0 etc. :type <float>
  :param item_id: id or name of an Exportable :type <str>
  :returns: list of exported item paths
  '''
  cmd.extend([src_path])

  if not dest_dir:
    dest_dir = mkdtemp(prefix='pyglass')

  cmd.extend(['--output=%s' % dest_dir])

  if export_format:
    cmd.extend(['--formats=%s' % export_format])

  if scale:
    cmd.extend(['--scales=%s' % scale])

  if item_id:
    cmd.extend(['--items=%s' % item_id])

  logger.debug(u'Executing cmd: %s' % cmd)
  exported_str = execute(cmd)
  logger.debug(u'Raw result: %s' % exported_str)
  # Raw result is in the form: 'Exported <item-name-1>\nExported <item-name-2>\n'

  exported_items = [os.path.join(dirpath, f)
                    for dirpath, dirnames, files in os.walk(dest_dir)
                    for f in files if f.endswith('.%s' % export_format)]

  return exported_items


def export_slices(*args, **kwargs):
  cmd = [SKETCHTOOL, 'export', 'slices']
  return export_cmd(cmd, *args, **kwargs)


def export_artboards(*args, **kwargs):
  cmd = [SKETCHTOOL, 'export', 'artboards']
  return export_cmd(cmd, *args, **kwargs)


def export_pages(*args, **kwargs):
  ''' Exports pages from src_path in dest_dir in given format and scale.

  >>> export_pages('~/example.sketch', dest_dir='~/Desktop/',
                    export_format=ExportFormat.PNG, scale=1.0)
  '''
  cmd = [SKETCHTOOL, 'export', 'pages']
  return export_cmd(cmd, *args, **kwargs)



# -*- coding: utf-8 -*-


def parse_slices(filename, page_dict):
  from .models import Slice
  slices = []
  if 'slices' in page_dict:
    for slice_dict in page_dict['slices']:
      slices.append(Slice(filename, slice_dict))
  return slices


def parse_artboards(filename, page_dict):
  from .models import Artboard
  artboards = []
  if 'artboards' in page_dict:
    for artboard_dict in page_dict['artboards']:
      artboards.append(Artboard(filename, artboard_dict))
  return artboards


def parse_pages(filename, list_dict):
  from .models import Page
  pages = []
  if 'pages' in list_dict:
    for page_dict in list_dict['pages']:
      pages.append(Page(filename, page_dict))
  return pages



# -*- coding: utf-8 -*-
# Default libs
import shutil

from tempfile import NamedTemporaryFile, mkdtemp
from os.path import isdir, exists, join, basename
from glob import glob

# Library modules
from pxprocess import check_call

# Project modules
from ..settings import QLMANAGE, QUICKGLASS
from ..utils import extension
from ..models import ExportFormat


############################################################
# DONT CALL DIRECTLY
############################################################
def embedded_preview(src_path):
  ''' Returns path to temporary copy of embedded QuickLook preview, if it exists '''
  try:
    assert(exists(src_path) and isdir(src_path))

    preview_list = glob(join(src_path, '[Q|q]uicklook', '[P|p]review.*'))
    assert(preview_list)  # Assert there's at least one preview file
    preview_path = preview_list[0]  # Simplistically, assume there's only one

    with NamedTemporaryFile(prefix='pyglass', suffix=extension(preview_path), delete=False) as tempfileobj:
      dest_path = tempfileobj.name
      shutil.copy(preview_path, dest_path)

    assert(exists(dest_path))
    return dest_path
  except:
    return None


def generator_preview(src_path):
  ''' Returns path to the preview created by the generator '''
  try:
    assert(exists(src_path))

    dest_dir = mkdtemp(prefix='pyglass')
    cmd = [QLMANAGE, '-p', src_path, '-o', dest_dir]
    assert(check_call(cmd) == 0)

    src_filename = basename(src_path)
    dest_list = glob(join(dest_dir, '%s.qlpreview' % (src_filename), '[P|p]review.*'))
    assert(dest_list)

    dest_path = dest_list[0]
    assert(exists(dest_path))

    return dest_path
  except:
    return None


def thumbnail_preview(src_path):
  ''' Returns the path to small thumbnail preview. '''
  try:
    assert(exists(src_path))

    max_width, max_height = '2640', '1520'
    export_format = ExportFormat.PNG

    with NamedTemporaryFile(prefix='pyglass', suffix='.png', delete=False) as tempfileobj:
      dest_path = tempfileobj.name

    cmd = [QUICKGLASS, '-srcPath', src_path, '-destPath', dest_path,
           '-maxWidth', max_width, '-maxHeight', max_height,
           '-exportFormat', export_format]
    assert(check_call(cmd) == 0)

    assert(exists(dest_path))
    return dest_path
  except:
    return None



# -*- coding: utf-8 -*-
# Default libs
import logging
import os

# Library modules
from pxprocess import check_output
from pyunicode import safely_decode

logger = logging.getLogger(__name__)


def execute(cmd):
  ''' Call cmd and return output. return None if any exception occurs '''
  try:
    return safely_decode(check_output(cmd))
  except Exception as e:
    logger.warn(u'Couldnt execute cmd: %s.\nReason: %s' % (cmd, e))
    return None


def unicode_or_none(dictionary, key):
  if dictionary is None or key is None:
    return None
  return None if key not in dictionary or dictionary[key] is None else unicode(dictionary[key])


def extension(path_str):
  ''' Returns lowercased file extension for the path '''
  return os.path.splitext(path_str)[1].lower()


def mimetype(path_str):
  ''' Returns the mimetype of the file at path_str. Depends on OS X's `file` util '''
  return execute(['file', '--mime-type', '--brief', path_str]).strip().lower()



# -*- coding: utf-8 -*-

# Project modules
from ..utils import unicode_or_none
from ..models import Exportable, GenericObject, ExportFormat

from .parse import parse_artboards, parse_slices


############################################################
# DIMENSIONS
############################################################
class Bounds(GenericObject):
  def __init__(self, bounds_str):
    bounds_list = [float(num) for num in bounds_str.split(',')]

    self.x, self.y = bounds_list[0], bounds_list[1]
    self.width, self.height = bounds_list[2], bounds_list[3]

  def __unicode__(self):
    return u'<Bounds (x=%s, y=%s, width=%s, height=%s)>' % (self.x, self.y, self.width, self.height)


class Rect(GenericObject):
  def __init__(self, rect_dict):
    self.x, self.y = float(rect_dict['x']), float(rect_dict['y'])
    self.width, self.height = float(rect_dict['width']), float(rect_dict['height'])

  def __unicode__(self):
    return u'<Rect (x=%s, y=%s, width=%s, height=%s)>' % (self.x, self.y, self.width, self.height)


############################################################
# EXPORTABLE OBJECTS
############################################################
class SketchExportable(Exportable):
  ''' Base class for any exportable Sketch item, i.e. Pages, Artboards, Slices '''
  def __init__(self, filename, exportable_dict):
    self.id = unicode_or_none(exportable_dict, 'id')
    self.name = unicode_or_none(exportable_dict, 'name')
    self.filename = filename  # The file we're exporting from
    super(SketchExportable, self).__init__()

  def __unicode__(self):
    return u'<SketchExportable (id="%s", name="%s")>' % (self.id, self.name)


class Page(SketchExportable):
  def __init__(self, filename, page_dict):
    self.bounds = Bounds(unicode_or_none(page_dict, 'bounds'))
    self.slices = parse_slices(filename, page_dict)
    self.artboards = parse_artboards(filename, page_dict)
    super(Page, self).__init__(filename, page_dict)

  def export(self, export_format=ExportFormat.PNG):
    from .export import export_pages
    pages = export_pages(self.filename, item_id=self.id, export_format=export_format)
    return pages[0] if pages else None

  def __unicode__(self):
    return u'<Page (id="%s", name="%s", bounds=%s, slices=%s, artboards=%s)>' % \
           (self.id, self.name, self.bounds, unicode(self.slices), unicode(self.artboards))


class Slice(SketchExportable):
  def __init__(self, filename, slice_dict):
    self.rect = Rect(slice_dict['rect'])
    super(Slice, self).__init__(filename, slice_dict)

  def export(self, export_format=ExportFormat.PNG):
    from .export import export_slices
    slices = export_slices(self.filename, item_id=self.id, export_format=export_format)
    return slices[0] if slices else None

  def __unicode__(self):
    return u'<Slice (id="%s", name="%s", rect=%s)>' % (self.id, self.name, self.rect)


class Artboard(SketchExportable):
  def __init__(self, filename, artboard_dict):
    self.rect = Rect(artboard_dict['rect'])
    super(Artboard, self).__init__(filename, artboard_dict)

  def export(self, export_format=ExportFormat.PNG):
    from .export import export_artboards
    artboards = export_artboards(self.filename, item_id=self.id, export_format=export_format)
    return artboards[0] if artboards else None

  def __unicode__(self):
    return u'<Artboard (id="%s", name="%s", rect=%s)>' % (self.id, self.name, self.rect)



# -*- coding: utf-8 -*-
# Default libs
import json
import logging

# Project modules
from ..settings import SKETCHTOOL
from ..utils import execute, extension
from .parse import parse_pages

logger = logging.getLogger(__name__)


def is_sketchfile(src_path):
  ''' Returns True if src_path is a sketch file '''
  if extension(src_path) == u'.sketch':
    return True
  return False


############################################################
# LIST COMMANDS - PASSTHROUGH TO SKETCHTOOL LIST
############################################################
def list_cmd(cmd, src_path):
  ''' Executes a `sketchtool list` command and parse the output
  :cmd: A sketchtool list command :type <list>
  :src_path: File to export. :type <str>
  :returns: A list of pages. Artboards & slices are included in the page hierarchy
  '''
  cmd.extend([src_path])

  logger.debug(u'Executing cmd: %s' % cmd)
  result = execute(cmd)
  if not result:
    return None

  logger.debug(u'Raw result: %s' % result)
  list_dict = json.loads(result)
  pages = parse_pages(src_path, list_dict)
  return pages


def list_slices(src_path):
  cmd = [SKETCHTOOL, 'list', 'slices']
  return list_cmd(cmd, src_path)


def list_artboards(src_path):
  cmd = [SKETCHTOOL, 'list', 'artboards']
  return list_cmd(cmd, src_path)


def list_pages(src_path):
  cmd = [SKETCHTOOL, 'list', 'pages']
  return list_cmd(cmd, src_path)


############################################################
# RETURNS PAGES, ARTBOARDS, SLICES WITH EXPORTED PNGS
############################################################
def pages(src_path):
  ''' Return pages as flat list '''
  pages = list_pages(src_path)
  return pages


def slices(src_path):
  ''' Return slices as a flat list '''
  pages = list_slices(src_path)
  slices = []
  for page in pages:
    slices.extend(page.slices)
  return slices


def artboards(src_path):
  ''' Return artboards as a flat list '''
  pages = list_artboards(src_path)
  artboards = []
  for page in pages:
    artboards.extend(page.artboards)
  return artboards


############################################################
# SIMPLE IMAGE PREVIEW OF FILE
############################################################
def preview(src_path):
  ''' Generates a preview of src_path as PNG.
  :returns: A list of preview paths, one for each page.
  '''
  previews = []
  for page in list_artboards(src_path):
    previews.append(page.export())
    for artboard in page.artboards:
      previews.append(artboard.export())
  return previews



# -*- coding: utf-8 -*-
# Default libs
import os
import stat

from os.path import dirname, join, abspath

curr_dir = dirname(abspath(__file__))


def make_executable(path_str):
  ''' Performs the equivalent of `chmod +x` on the file at path_str.
  :returns: path_str if success, else None
  '''
  try:
    mode = os.stat(path_str).st_mode
    os.chmod(path_str, mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
  except Exception as e:
    print 'Exception: %s' % e
    return None
  return path_str


def sketchtool_executable():
  make_executable(join(curr_dir, 'lib', 'SketchTool', 'sketchmigrate'))
  return make_executable(join(curr_dir, 'lib', 'SketchTool', 'sketchtool'))

def quickglass_executable():
  return make_executable(join(curr_dir, 'lib', 'QuickGlass'))


SKETCHTOOL = sketchtool_executable()
QUICKGLASS = quickglass_executable()
QLMANAGE = join('/usr', 'bin', 'qlmanage')



# -*- coding: utf-8 -*-
# Default libs
import os

from tempfile import NamedTemporaryFile
from os.path import exists

# Library modules
from PyPDF2 import PdfFileMerger, PdfFileReader, PdfFileWriter
from pxprocess import check_call


############################################################
# PDF CLASSES
############################################################
def stitch_pdfs(pdf_list):
  ''' Merges a series of single page pdfs into one multi-page doc '''
  pdf_merger = PdfFileMerger()
  for pdf in pdf_list:
    pdf_merger.append(pdf)

  with NamedTemporaryFile(prefix='pyglass', suffix='.pdf', delete=False) as tempfileobj:
    dest_path = tempfileobj.name

  pdf_merger.write(dest_path)
  pdf_merger.close()
  return dest_path


def split_pdf(pdf_path):
  ''' Splits a multi-page pdf into a list of single page pdfs '''
  pdf = PdfFileReader(pdf_path)
  pdf_list = []

  for page_num in range(pdf.numPages):
    page = pdf.getPage(page_num)

    pdf_writer = PdfFileWriter()
    pdf_writer.addPage(page)

    with NamedTemporaryFile(prefix='pyglass', suffix='.pdf', delete=False) as tempfileobj:
      pdf_writer.write(tempfileobj)
      page_path = tempfileobj.name

    pdf_list.append(page_path)

  return pdf_list


def to_png(pdf_path):
  ''' Converts a single-page pdf to a png image via the `sips` command
  :returns: Path to the converted png
  '''
  try:
    with NamedTemporaryFile(prefix='pyglass', suffix='.png', delete=False) as tempfileobj:
      png_path = tempfileobj.name

    cmd = ['sips', '-s', 'format', 'png', pdf_path, '--out', png_path]
    assert(check_call(cmd) == 0)

    assert(exists(png_path))
    return png_path
  except:
    return None


def to_pngs(pdf_path):
  ''' Converts a multi-page pdfs to a list of pngs via the `sips` command
  :returns: A list of converted pngs
  '''
  pdf_list = split_pdf(pdf_path)
  pngs = []
  for pdf in pdf_list:
    pngs.append(to_png(pdf))
    os.remove(pdf)  # Clean up
  return pngs



# -*- coding: utf-8 -*-
# Default libs

# Installed libs

# Project modules
from ..models import ExportMimeType
from ..pdf import to_pngs
from ..utils import mimetype
from .export import embedded_preview, generator_preview, thumbnail_preview


def preview(src_path):
  ''' Generates a preview of src_path in the requested format.
  :returns: A list of preview paths, one for each page. Blank list if unsupported.
  '''
  preview = embedded_preview(src_path)

  if not preview:
    preview = generator_preview(src_path)

  if not preview:
    preview = thumbnail_preview(src_path)

  # Ensure the preview is returned in the right format
  if preview:
    if mimetype(preview) in [ExportMimeType.PNG]:
      return [preview]

    if mimetype(preview) in [ExportMimeType.PDF]:
      return to_pngs(preview)

  return []



# -*- coding: utf-8 -*-
# Library modules
from pyunicode import safely_decode

# Project modules
from . import quicklook, sketch

def preview(src_path):
  ''' Generates a preview of src_path in the requested format.
  :returns: A list of preview paths, one for each page.
  '''
  previews = []

  if sketch.is_sketchfile(src_path):
    previews = sketch.preview(src_path)

  if not previews:
    previews = quicklook.preview(src_path)

  previews = [safely_decode(preview) for preview in previews]

  return previews



#!/usr/bin/env python

# System modules
import sys
import os
import shlex
import subprocess
import shutil
import platform

from os.path import join

# Library modules
try:
  from setuptools import setup, find_packages
except ImportError:
  from distutils.core import setup

from distutils.dir_util import copy_tree
from distutils.file_util import copy_file

# Package modules
if sys.version_info[:2] < (2, 7):
  print "Sorry, pyglass requires python version 2.7 or later"
  sys.exit(1)

if platform.system() != 'Darwin':
  print "Sorry, pyglass only runs on OS X"
  sys.exit(1)


class Dir:
  BUILD = 'build'
  DIST = 'dist'
  COCOA = 'cocoa'
  COCOA_BUILD = join(COCOA, 'build')
  LIB = join('pyglass', 'lib')  # Destination directory for vendor/custom libs
  VENDOR = 'vendor'  # Third-party libraries


def rm_tempdirs():
  ''' Remove temporary build folders '''
  tempdirs = [Dir.BUILD, Dir.COCOA_BUILD, Dir.LIB]
  for tempdir in tempdirs:
    if os.path.exists(tempdir):
      shutil.rmtree(tempdir, ignore_errors=True)


def xcodebuild():
  ''' Build the QuickGlass binary in Release mode '''
  # Build from xcodeproj
  os.chdir(Dir.COCOA)
  cmd = 'xcodebuild build'
  subprocess.call(shlex.split(cmd))
  os.chdir('..')


def copy_vendor_libs():
  ''' Copies third party vendor libs into the module '''
  copy_tree('%s/' % Dir.VENDOR, '%s/' % Dir.LIB)


def copy_custom_libs():
  ''' Copies custom build libs into the module '''
  copy_file('%s/Release/QuickGlass' % Dir.COCOA_BUILD, '%s/QuickGlass' % Dir.LIB)


def lib_list():
  ''' Returns the contents of 'pyglass/lib' as a list of 'lib/*' items for package_data '''
  lib_list = []
  for (root, dirs, files) in os.walk(Dir.LIB):
    for filename in files:
      root = root.replace('pyglass/', '')
      lib_list.append(join(root, filename))
  return lib_list

# Compile custom project
rm_tempdirs()
xcodebuild()

# Copy over libs into Dir.LIB
os.makedirs(Dir.LIB)
copy_custom_libs()
copy_vendor_libs()

package_libs = lib_list()

version = '0.1.2'

setup(
  name='pyglass',
  version=version,
  url='http://github.com/Pixelapse/pyglass',
  download_url='https://github.com/Pixelapse/pyglass/tarball/v%s' % version,
  description='Mac OS X File Preview Generator',
  long_description=open('README.md').read(),
  author='Shravan Reddy',
  author_email='shravan@pixelapse.com',
  maintainer='Pixelapse',
  maintainer_email='hello@pixelapse.com',
  packages=find_packages(),
  package_data={'': package_libs},
  install_requires=['pxprocess', 'pyunicode', 'PyPDF2'],
  include_package_data=True,
  zip_safe=False,
  license=open('LICENSE').read()
)

rm_tempdirs()



# -*- coding: utf-8 -*-

"""
pyglass library
~~~~~~~~~~~~~~~~~~~~~

pyglass is a simple wrapper around the cocoa project QuickGlass to extract
QuickLook previews from files.

Basic usage:

   >>> import pyglass
   >>> previews = pyglass.preview('design_v1.sketch')
   >>> previews
   ['/var/folders/fq/xtn_qh1x6c3drpp3ycytx1fr0000gn/T/pyglassY92Xqs',
   '/var/folders/fq/xtn_qh1x6c3drpp3ycytx1fr0000gn/T/pyglassZ34Jab']

:copyright: (c) 2014 by Pixelapse.
:license: MIT, see LICENSE for more details.

"""

__title__ = 'pyglass'
__author__ = 'Shravan Reddy'
__license__ = 'MIT'
__copyright__ = 'Copyright 2014 Pixelapse'

from .api import preview



# -*- coding: utf-8 -*-

# Project modules
from ..models import Exportable, ExportFormat


class QLExportable(Exportable):
  ''' Base class for any exportable QuickLook item '''
  def __init__(self, filename):
    self.filename = filename
    super(QLExportable, self).__init__()

  def __unicode__(self):
    return u'<QLExportable>'


def Page(QLExportable):
  ''' For multi-page files, e.g. if pdf preview '''
  def __init__(self, filename, page_id):
    self.id = page_id
    super(Page, self).__init__(filename)

  def export(self, export_format=ExportFormat.PNG):
    pass
